import os
import textwrap
from typing import Optional

from cpl_core.database import DatabaseSettings
from cpl_core.database.context import DatabaseContextABC

from migration_to_sql.mock.mock_cursor import MockCursor


class MockDBContext(DatabaseContextABC):
    def __init__(self):
        DatabaseContextABC.__init__(self)
        self._migration_version: Optional[str] = None
        self._migration_name: Optional[str] = None
        self._migration_script: Optional[str] = None

    @property
    def cursor(self) -> MockCursor:
        cursor = MockCursor()
        cursor.execute = self.execute
        return cursor

    def set_migration(self, name: str, upgrade: bool = False):
        self._migration_name = f'{name.split("_")[1].replace("Migration", "")}_{"up" if upgrade else "down"}'
        self._migration_version = name.split("_")[0]
        self._migration_script = ""

    def connect(self, database_settings: DatabaseSettings):
        pass

    def select(self, statement: str) -> list[tuple]:
        pass

    def execute(self, query: str, *args, **kwargs):
        if "MigrationHistory" in query:
            return None
        else:
            self._migration_script += f"{textwrap.dedent(query)}\n\n"

    def save_changes(self):
        path = f"../../src/bot_data/scripts/{self._migration_version}"
        if not os.path.exists(path):
            os.makedirs(path)

        script = textwrap.dedent(self._migration_script)
        with open(f"{path}/{self._migration_name}.sql", "w+") as f:
            f.write(script)
            f.close()

        self._migration_name = None
        self._migration_version = None
        self._migration_script = None
